@@ -15,35 +15,33 @@
 
 These directives are specific to Five and have no equivalents in Zope 3.
 
-$Id: fiveconfigure.py 68848 2006-06-26 17:40:26Z tseaver $
+$Id: fiveconfigure.py 72986 2007-03-05 13:07:27Z rocky $
 """
 import os
-import sys
 import glob
 import warnings
 import logging
-logger = logging.getLogger('Five')
 
 import App.config
+from App.Product import initializeProduct
+from App.ProductContext import ProductContext
 import Products
+import Zope2
 
-from zope.interface import classImplements, classImplementsOnly, implementedBy
-from zope.interface.interface import InterfaceClass
+from zope.interface import classImplements, implementedBy
+from zope.component import getUtility
+from zope.component.interface import provideInterface
 from zope.configuration import xmlconfig
 from zope.configuration.exceptions import ConfigurationError
 from zope.publisher.interfaces.browser import IDefaultBrowserLayer
+from zope.security.interfaces import IPermission
 
-from zope.app import zapi
-from zope.app.component.interface import provideInterface
-from zope.app.component.metaconfigure import adapter
-from zope.app.security.interfaces import IPermission
-
-from Products.Five.viewable import Viewable
-from Products.Five.traversable import Traversable
+from Products.Five import isFiveMethod
 from Products.Five.bridge import fromZ2Interface
 from Products.Five.browser.metaconfigure import page
 
 debug_mode = App.config.getConfiguration().debug_mode
+LOG = logging.getLogger('Five')
 
 def findProducts():
     import Products
@@ -63,35 +61,30 @@ def handleBrokenProduct(product):
     # XXX It would be really cool if we could make this product appear broken
     # in the control panel. However, all attempts to do so has failed from my 
     # side. //regebro
-    exc = sys.exc_info()
-    logger.error('Could not import Product %s' % product.__name__, error=exc)
+    LOG.error('Could not import Product %s' % product.__name__, exc_info=True)
 
-def loadProducts(_context):
-    products = findProducts()
+def loadProducts(_context, file=None):
+    if file is None:
+        # set the default
+        file = 'configure.zcml'
     
-    # first load meta.zcml files
-    for product in products:
-        zcml = os.path.join(os.path.dirname(product.__file__), 'meta.zcml')
-        if os.path.isfile(zcml):
-            try:
-                xmlconfig.include(_context, zcml, package=product)
-            except: # Yes, really, *any* kind of error.
-                handleBrokenProduct(product)
-                
-    # now load their configure.zcml
-    for product in products:
-        zcml = os.path.join(os.path.dirname(product.__file__),
-                            'configure.zcml')
+    # now load the files if they exist
+    for product in findProducts():
+        zcml = os.path.join(os.path.dirname(product.__file__), file)
         if os.path.isfile(zcml):
             try:
                 xmlconfig.include(_context, zcml, package=product)
             except: # Yes, really, *any* kind of error.
                 handleBrokenProduct(product)
 
-def loadProductsOverrides(_context):
+def loadProductsOverrides(_context, file=None):
+    if file is None:
+        # set the default
+        file = 'overrides.zcml'
+    
+    # now load the files if they exist
     for product in findProducts():
-        zcml = os.path.join(os.path.dirname(product.__file__),
-                            'overrides.zcml')
+        zcml = os.path.join(os.path.dirname(product.__file__), file)
         if os.path.isfile(zcml):
             try:
                 xmlconfig.includeOverrides(_context, zcml, package=product)
@@ -112,76 +105,21 @@ def implements(_context, class_, interface):
                     interface)
             )
 
-def isFiveMethod(m):
-    return hasattr(m, '__five_method__')
-
-_traversable_monkies = []
-def classTraversable(class_):
-    # If a class already has this attribute, it means it is either a
-    # subclass of Traversable or was already processed with this
-    # directive; in either case, do nothing... except in the case were
-    # the class overrides __bobo_traverse__ instead of getting it from
-    # a base class. In this case, we suppose that the class probably
-    # didn't bother with the base classes __bobo_traverse__ anyway and
-    # we step __fallback_traverse__.
-    if hasattr(class_, '__five_traversable__'):
-        if (hasattr(class_, '__bobo_traverse__') and
-            isFiveMethod(class_.__bobo_traverse__)):
-            return
-
-    if (hasattr(class_, '__bobo_traverse__') and
-        not isFiveMethod(class_.__bobo_traverse__)):
-        # if there's an existing bobo_traverse hook already, use that
-        # as the traversal fallback method
-        setattr(class_, '__fallback_traverse__', class_.__bobo_traverse__)
-
-    setattr(class_, '__bobo_traverse__',
-            Traversable.__bobo_traverse__.im_func)
-    setattr(class_, '__five_traversable__', True)
-    # remember class for clean up
-    _traversable_monkies.append(class_)
-
+# BBB 2006/05/01 -- to be removed after 12 months
 def traversable(_context, class_):
-    _context.action(
-        discriminator = None,
-        callable = classTraversable,
-        args = (class_,)
-        )
-
-_defaultviewable_monkies = []
-def classDefaultViewable(class_):
-    # If a class already has this attribute, it means it is either a
-    # subclass of DefaultViewable or was already processed with this
-    # directive; in either case, do nothing... except in the case were
-    # the class overrides the attribute instead of getting it from
-    # a base class. In this case, we suppose that the class probably
-    # didn't bother with the base classes attribute anyway.
-    if hasattr(class_, '__five_viewable__'):
-        if (hasattr(class_, '__browser_default__') and
-            isFiveMethod(class_.__browser_default__)):
-            return
-
-    if hasattr(class_, '__browser_default__'):
-        # if there's an existing __browser_default__ hook already, use that
-        # as the fallback
-        if not isFiveMethod(class_.__browser_default__):
-            setattr(class_, '__fallback_default__', class_.__browser_default__)
-    if not hasattr(class_, '__fallback_default__'):
-        setattr(class_, '__fallback_default__',
-                Viewable.__fallback_default__.im_func)
-
-    setattr(class_, '__browser_default__',
-            Viewable.__browser_default__.im_func)
-    setattr(class_, '__five_viewable__', True)
-    # remember class for clean up
-    _defaultviewable_monkies.append(class_)
+    warnings.warn("The five:traversable statement is no longer needed "
+                  "and will be removed in Zope 2.12.",
+                  DeprecationWarning, 2)
 
+# BBB 2006/05/01 -- to be removed after 12 months
 def defaultViewable(_context, class_):
-    _context.action(
-        discriminator = None,
-        callable = classDefaultViewable,
-        args = (class_,)
-        )
+    warnings.warn("The five:defaultViewable statement is no longer "
+                  "needed and will be removed in Zope 2.12. \n If you rely "
+                  "on it to make 'index.html' the default view, replace it "
+                  "with <browser:defaultView name='index.html' />",
+                  DeprecationWarning, 2)
+    from Products.Five.bbb import IBrowserDefault
+    implements(_context, class_, (IBrowserDefault,))
 
 def createZope2Bridge(zope2, package, name):
     # Map a Zope 2 interface into a Zope3 interface, seated within 'package'
@@ -231,7 +169,7 @@ _meta_type_regs = []
 def _registerClass(class_, meta_type, permission, addview, icon, global_):
     setattr(class_, 'meta_type', meta_type)
 
-    permission_obj = zapi.getUtility(IPermission, permission)
+    permission_obj = getUtility(IPermission, permission)
 
     if icon:
         setattr(class_, 'icon', '++resource++%s' % icon)
@@ -260,6 +198,48 @@ def registerClass(_context, class_, meta_type, permission, addview=None,
         args = (class_, meta_type, permission, addview, icon, global_)
         )
 
+def _registerPackage(module_, init_func=None):
+    """Registers the given python package as a Zope 2 style product
+    """
+
+    if not hasattr(module_, '__path__'):
+        raise ValueError("Must be a package and the " \
+                         "package must be filesystem based")
+    
+    app = Zope2.app()
+    try:
+        product = initializeProduct(module_, 
+                                    module_.__name__, 
+                                    module_.__path__[0],
+                                    app)
+
+        product.package_name = module_.__name__
+
+        if init_func is not None:
+            newContext = ProductContext(product, app, module_)
+            init_func(newContext)
+
+        registered_packages = getattr(Products, '_registered_packages', None)
+        if registered_packages is None:
+            registered_packages = Products._registered_packages = []
+        registered_packages.append(module_)
+    finally:
+        try:
+            import transaction
+            transaction.commit()
+        finally:
+            app._p_jar.close()
+
+def registerPackage(_context, package, initialize=None):
+    """ZCML directive function for registering a python package product
+    """
+
+    _context.action(
+        discriminator = ('registerPackage', package),
+        callable = _registerPackage,
+        args = (package,initialize)
+        )
+
 # clean up code
 
 def killMonkey(class_, name, fallback, attr=None):
@@ -283,17 +263,6 @@ def killMonkey(class_, name, fallback, attr=None):
         except (AttributeError, KeyError):
             pass
 
-def untraversable(class_):
-    """Restore class's initial state with respect to traversability"""
-    killMonkey(class_, '__bobo_traverse__', '__fallback_traverse__',
-               '__five_traversable__')
-
-def undefaultViewable(class_):
-    """Restore class's initial state with respect to being default
-    viewable."""
-    killMonkey(class_, '__browser_default__', '__fallback_default__',
-               '__five_viewable__')
-
 def unregisterClass(class_):
     delattr(class_, 'meta_type')
     try:
@@ -302,15 +271,6 @@ def unregisterClass(class_):
         pass
 
 def cleanUp():
-    global _traversable_monkies
-    for class_ in _traversable_monkies:
-        untraversable(class_)
-    _traversable_monkies = []
-
-    global _defaultviewable_monkies
-    for class_ in _defaultviewable_monkies:
-        undefaultViewable(class_)
-    _defaultviewable_monkies = []
 
     global _register_monkies
     for class_ in _register_monkies:
