@@ -12,7 +12,7 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: HTTPResponse.py 68779 2006-06-21 16:48:26Z tseaver $'''
+$Id: HTTPResponse.py 70564 2006-10-08 08:30:16Z andreasjung $'''
 __version__ = '$Revision: 1.81 $'[11:-2]
 
 import types, os, sys, re
@@ -343,7 +343,7 @@ class HTTPResponse(BaseResponse):
             self.setHeader('content-type', c)
         else:
             c = self.headers['content-type']
-            if not 'charset=' in  c:
+            if c.startswith('text/') and not 'charset=' in  c:
                 c = '%s; charset=%s' % (c, default_encoding)                
                 self.setHeader('content-type', c)
 
@@ -445,13 +445,26 @@ class HTTPResponse(BaseResponse):
                                               r'charset=([-_0-9a-z]+' +
                                               r')(?:(?:\s*;)|\Z)',
                                               re.IGNORECASE)):
+
+        def fix_xml_preamble(body, encoding):
+            """ fixes the encoding in the XML preamble according
+                to the charset specified in the content-type header.
+            """
+
+            if body.startswith('<?xml'):
+                pos_right = body.find('?>')  # right end of the XML preamble
+                body = ('<?xml version="1.0" encoding="%s" ?>' % encoding) + body[pos_right+2:]
+            return body
+
         # Encode the Unicode data as requested
 
         if self.headers.has_key('content-type'):
             match = charset_re.match(self.headers['content-type'])
             if match:
                 encoding = match.group(1)
-                return body.encode(encoding)
+                body = body.encode(encoding)
+                body = fix_xml_preamble(body, encoding)
+                return body
             else:
 
                 ct = self.headers['content-type']
@@ -459,7 +472,9 @@ class HTTPResponse(BaseResponse):
                     self.headers['content-type'] = '%s; charset=%s' % (ct, default_encoding)
 
         # Use the default character encoding
-        return body.encode(default_encoding,'replace')
+        body = body.encode(default_encoding,'replace')
+        body = fix_xml_preamble(body, default_encoding)
+        return body
 
     def setBase(self,base):
         """Set the base URL for the returned document.
