@@ -13,7 +13,7 @@
 ##############################################################################
 """Unauthorized Exception
 
-$Id: unauthorized.py 26816 2004-07-28 19:09:51Z pruggera $
+$Id: unauthorized.py 69465 2006-08-14 12:49:46Z ctheune $
 """
 __docformat__ = 'restructuredtext'
 
@@ -29,7 +29,7 @@ class Unauthorized(object):
         self.request = request
 
     def __call__(self):
-        self.request.unauthorized("basic realm='Zope'")
+        self.request.unauthorized('basic realm="Zope"')
         return ''
 
     __str__ = __call__
